/*
pxlrace.c - Pixel Race For Windows 9x
Version 1.0.0
Copyright (C) 2003-2004 dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

    Free Software Foundation, Inc.
    59 Temple Place - Suite 330
    Boston, MA  02111-1307, USA.
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "rnd.h"

#define ID_TIMER 1
#define MAX_X 610
#define MAX_Y 375
/* values for iReset */
#define FIRST    1
#define RESET    2
#define FINISHED 3

int iReset;
int iWinner,iLeadPos,iHeatTime;
int iPrevWinner,iPrevLeadPos;
int iRacer[MAX_Y];
int iTopTime[11];
int iTopWinner[11];
int cxChar,cyChar,cxCaps;

/*******************************************************************
char szDbg1[80];
char szDbg2[80];
char szDbg3[80];
*******************************************************************/

LRESULT CALLBACK WndProc   (HWND, UINT, WPARAM, LPARAM);
VOID    CALLBACK TimerProc (HWND, UINT, UINT,   DWORD);

int WINAPI WinMain (HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    PSTR szCmdLine,
                    int iCmdShow)
   {
   static char szAppName[] = "PopClk";
   HWND hwnd;
   MSG msg;
   WNDCLASSEX wndclass;

   wndclass.cbSize         = sizeof(wndclass);
   wndclass.style          = CS_HREDRAW | CS_VREDRAW;
   wndclass.lpfnWndProc    = WndProc;
   wndclass.cbClsExtra     = 0;
   wndclass.cbWndExtra     = 0;
   wndclass.hInstance      = hInstance;
   wndclass.hIcon          = LoadIcon (hInstance, "pxlrace");
   wndclass.hCursor        = LoadCursor (NULL, IDC_ARROW);
   wndclass.hbrBackground  = (HBRUSH)  GetStockObject (WHITE_BRUSH);
   wndclass.lpszMenuName   = NULL;
   wndclass.lpszClassName  = szAppName;
   wndclass.hIconSm        = LoadIcon (hInstance, "pxlrace");

   RegisterClassEx (&wndclass);

   hwnd = CreateWindow (szAppName,  // Window Class Name
      "Pixel Race",                 // Window Caption
      WS_OVERLAPPEDWINDOW,          // Window Style
      10,                           // initial x position
      10,                           // initial y position
      MAX_X+10,                     // initial x size
      MAX_Y+50,                     // initial y size
      NULL,                         // parent window handle
      NULL,                         // window menu handle
      hInstance,                    // program instance handle
      NULL);                        // creation parameters


   { /* set timer */
   int iResult;
   iResult = SetTimer(hwnd, ID_TIMER, 1,
                     (TIMERPROC) TimerProc);
   if (!iResult)
      {
      MessageBox(hwnd,
         "Too many clocks or timers!",
         "Pixel Race",
         MB_ICONEXCLAMATION | MB_OK);
      return FALSE;
      } /* too many timers */
   } /* set timer */

   ShowWindow (hwnd, iCmdShow);
   UpdateWindow (hwnd);

   while (GetMessage (&msg, NULL, 0, 0))
      {
      TranslateMessage (&msg);
      DispatchMessage (&msg);
      } // while (GetMessage (&msg, NULL, 0, 0))

   return msg.wParam;
   } /* WinMain */

LRESULT CALLBACK WndProc (HWND hwnd,
                          UINT iMsg,
                          WPARAM wParam,
                          LPARAM lParam)
   {
   PAINTSTRUCT ps;
   switch (iMsg)
      {
      case WM_CREATE:
         { /* initialize */
         int *p,*q;
         TEXTMETRIC tm;
         HDC hdc;
         rndSeed = (unsigned char *) rndinit();
         p = iRacer;
         q = p + MAX_Y;
         while (p < q)
            {
            *p++ = 0;
            } /* initialize iRacer */
         iReset   = FIRST;
         iWinner  = 0;
         iLeadPos = -999999;
         iHeatTime   = 0;
         p = iTopTime;
         q = p + 11;
         while (p < q) *p++ = 999999999;
         p = iTopWinner;
         q = p + 11;
         while (p < q) *p++ = 0;
         hdc = GetDC(hwnd);
         GetTextMetrics(hdc,&tm);
         cxChar = tm.tmAveCharWidth;
         cxCaps = (tm.tmPitchAndFamily & 1 ? 3 : 2)
            * (cxChar >> 1);
         cyChar = tm.tmHeight + tm.tmExternalLeading;
         ReleaseDC(hwnd,hdc);
         return 0;
         } /* initialize */

      case WM_PAINT:

         BeginPaint(hwnd,&ps);
         EndPaint(hwnd,&ps);
         iReset = RESET;

         return 0;

      case WM_LBUTTONDOWN:

         PostMessage(hwnd,WM_CLOSE,0,0L);

         return 0;

      case WM_KEYDOWN:

         if (wParam == 'Q')
            {
            PostMessage(hwnd,WM_CLOSE,0,0L);
            } /* if quit */

         else if (wParam == 'R')
            {
            iReset = RESET;
            } /* if another race */

         return 0;

      case WM_DESTROY:
         KillTimer(hwnd, ID_TIMER);
         PostQuitMessage (0);
         return 0;

      } /* switch (iMsg) */

   return DefWindowProc (hwnd, iMsg, wParam, lParam);
   } /* WndProc */


void CALLBACK TimerProc (HWND hwnd,
                         UINT iMsg,
                         UINT iTimerID,
                         DWORD dwTime)
   {
   HDC hdc;
   RECT rect;
   int i;
   int y;
   int *p,*q,*r,*s,*t;

   GetClientRect (hwnd, &rect);
   hdc = GetDC(hwnd);

   /* if finished, wait for reset */
   if (iReset == FINISHED)
      {
      ReleaseDC(hwnd, hdc);
      return;
      } /* if finished , wait for reset */

   if (iReset)
      {
      int iSeq,y;
      char szStr[128];
      p = iRacer;
      q = p + MAX_Y;
      y = 20;
      while (p < q)
         {
         if (iReset == RESET)
            {
            SetPixel(hdc,*p,y,RGB(255,255,255));
            } /* if not first race */
         *p++ = 0;
         SetPixel(hdc,0,y++,RGB(0,0,0));
         } /* line up at start */
      SetPixel(hdc,iPrevLeadPos,
         iPrevWinner+20,RGB(255,255,255));
      SetPixel(hdc,iPrevLeadPos-1,
         iPrevWinner+20,RGB(255,255,255));
      SetPixel(hdc,iPrevLeadPos-2,
         iPrevWinner+20,RGB(255,255,255));
      /* Erase Time */
      sprintf(szStr,"                                      ");
      TextOut(hdc,250,21,szStr,strlen(szStr));
      sprintf(szStr,"                                      ");
      TextOut(hdc,320,21,szStr,strlen(szStr));
      /* Erase top ten winners */
      iSeq = 0;
      y = 40;
      while (iSeq++ < 11)  /* including header */
         {
         sprintf(szStr,"                                    ");
         TextOut(hdc,20,y,szStr,strlen(szStr));
         y += 20;
         } /* erase top 10 winners */
      TextOut(hdc,20,320,"                              ", 30);
      TextOut(hdc,20,340,"                              ", 30);
      /*                  ....+....1....+....2....+....3  */
      iReset = 0;
      } /* if new race */

   for (i=0;i<4096;i++)
      {
      char szStr[128];
      y = rndnum(MAX_Y,rndSeed);
      p = iRacer + y;
      SetPixel(hdc,*p,y+20,RGB(255,255,255));
      *p = *p + 1;
      if (*p > iLeadPos)
         {
         int cxOfst;
         iWinner = y;
         iLeadPos = *p;
         sprintf(szStr,"                              ");
         TextOut(hdc,250,1,szStr,strlen(szStr));
         sprintf(szStr,"Leader");
         TextOut(hdc,250,1,szStr,strlen(szStr));
         SetTextAlign(hdc, TA_RIGHT | TA_TOP);
         cxOfst = 250 + (6 * cxChar) + (6 * cxChar);
         sprintf(szStr,"%6d", y+1);
         TextOut(hdc,cxOfst,1,szStr,strlen(szStr));
         SetTextAlign(hdc, TA_LEFT | TA_TOP);
         } /* new leader */

      if (iLeadPos >= MAX_X)
         {
         int iSeq,y;
         int iTempTime,iTempWinner;
         int iHoldTime,iHoldWinner;
         int cxOfst;
         r = iTopTime;
         s = r + 10;
         t = iTopWinner;
         while (r < s)
            {
            if (iHeatTime < *r)
               {
               break;
               } /* insertion point */
            r++;
            t++;
            } /* insert iHeatTime in iTopTime */
         if (r < s)
            {
            iTempTime   = *r;
            iTempWinner = *t;
            *r++ = iHeatTime;
            *t++ = iWinner+1;
            } /* if within top ten */
         while (r < s)
            {
            iHoldTime   = *r;
            iHoldWinner = *t;
            *r = iTempTime;
            *t = iTempWinner;
            iTempTime   = iHoldTime;
            iTempWinner = iHoldWinner;
            r++;
            t++;
            } /* shift iTopTime and iTopWinner to right */

         // iWinner = MAX_Y - 1;
         SetPixel(hdc,iLeadPos,  iWinner+20,RGB(255,0,0));
         SetPixel(hdc,iLeadPos-1,iWinner+20,RGB(255,0,0));
         SetPixel(hdc,iLeadPos-2,iWinner+20,RGB(255,0,0));
         sprintf(szStr,"                              ");
         TextOut(hdc,250,1,szStr,strlen(szStr));
         sprintf(szStr,"                                          ");
         TextOut(hdc,250,21,szStr,strlen(szStr));
         sprintf(szStr,"Winner");
         TextOut(hdc,250,1,szStr,strlen(szStr));
         SetTextAlign(hdc, TA_RIGHT | TA_TOP);
         sprintf(szStr,"%7d", iWinner+1);
         cxOfst = 250 + (6 * cxChar) + (7 * cxChar);
         TextOut(hdc,cxOfst,1,szStr,strlen(szStr));
         SetTextAlign(hdc, TA_LEFT | TA_TOP);
         sprintf(szStr,"Time");
         TextOut(hdc,250,21,szStr,strlen(szStr));
         SetTextAlign(hdc, TA_RIGHT | TA_TOP);
         cxOfst = 250 + (4 * cxChar) + (11 * cxChar);
         sprintf(szStr,"%11d", iHeatTime);
         TextOut(hdc,cxOfst,21,szStr,strlen(szStr));
         SetTextAlign(hdc, TA_LEFT | TA_TOP);
         sprintf(szStr,"      Player       Time");
         TextOut(hdc,20,40,szStr,strlen(szStr));
         r = iTopTime;
         s = r + 10;
         t = iTopWinner;
         iSeq = 1;
         y = 60;
         while (r < s)
            {
            if (*r == 999999999) break;
            SetTextAlign(hdc, TA_RIGHT | TA_TOP);
            sprintf(szStr,"%2d.", iSeq);
            cxOfst = 20 + (3 * cxChar);
            TextOut(hdc,cxOfst,y,szStr,strlen(szStr));
            sprintf(szStr,"%6d", *t);
            cxOfst = 20 + (3 * cxChar) + (6 * cxChar);
            TextOut(hdc,cxOfst,y,szStr,strlen(szStr));
            sprintf(szStr,"%10d", *r);
            cxOfst = 20 + (9 * cxChar) + (10 * cxChar);
            TextOut(hdc,cxOfst,y,szStr,strlen(szStr));
            SetTextAlign(hdc, TA_LEFT | TA_TOP);
            r++;
            t++;
            iSeq++;
            y += 20;
            } /* list top 10 winners */
         TextOut(hdc,20,320,"Press r to repeat", 17);
         TextOut(hdc,20,340,"Press q to quit", 15);
         /*                  ....+....1....+....2  */
         iPrevLeadPos = iLeadPos;
         iPrevWinner  = iWinner;
         iLeadPos = -999999;
         iWinner  = 0;
         iReset   = FINISHED;
         iHeatTime = 0;
         break;
         } /* if end of heat */
      else
         {
         SetPixel(hdc,*p,y+20,RGB(0,0,0));
         iHeatTime++;
         } /* still racing */
      } /* do MAX_Y pixels at a time */

   ReleaseDC(hwnd, hdc);
   } /* TimerProc */
